package jalview.hmmer;

import jalview.bin.Cache;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.AlignmentOrder;
import jalview.datamodel.HiddenMarkovModel;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.gui.Preferences;
import jalview.io.DataSourceType;
import jalview.io.FileFormat;
import jalview.io.FileLoader;
import jalview.util.MessageManager;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class HMMAlignThread implements Runnable
{

  AlignFrame af;

  AlignmentI alignment;

  HiddenMarkovModel hmm;

  boolean newFrame;

  long barID;

  Map<Integer, SequenceI> hmmSeqs;

  public HMMAlignThread(AlignFrame af, boolean createNewFrame)
  {
    this.af = af;
    alignment = af.getViewport().getAlignment();
    hmm = alignment.getSequenceAt(0).getHMM();
    newFrame = createNewFrame;
  }

  @Override
  public void run()
  {
    barID = System.currentTimeMillis();
    af.setProgressBar(MessageManager.getString("status.running_hmmbuild"),
            barID);
    HMMERCommands.HMMERFOLDER = Cache.getProperty(Preferences.HMMER_PATH);
    try
    {
      try
      {
        hmmSeqs = alignment.getHMMConsensusSequences(true);
        HMMERCommands.exportData(alignment, true, true, hmm);
      } catch (FileNotFoundException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();

      }
      try
      {
        runCommand();
      } catch (IOException | InterruptedException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      try
      {
        importData();
      } catch (IOException | InterruptedException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    } catch (Exception e)
    {
      e.printStackTrace();
    } finally
    {
      af.setProgressBar(MessageManager.getString("status.running_hmmalign"),
              barID);
    }
  }

  private void runCommand() throws IOException, InterruptedException
  {
    String command = HMMERCommands.HMMERFOLDER + HMMERCommands.HMMALIGN;
    if (!hmm.getFileHeader().contains("HMMER3/f"))
    {
      command += HMMERCommands.ALLCOL;
    }
    command += HMMERCommands.TRIM + HMMERCommands.OUTPUTALIGNMENT
            + HMMERCommands.JALVIEWDIRECTORY + HMMERCommands.HMMBUFFER
            + HMMERCommands.JALVIEWDIRECTORY
            + HMMERCommands.ALIGNMENTBUFFER;
    HMMERCommands.runCommand(command);
  }

  private void importData() throws IOException, InterruptedException
  {
    if (newFrame)
    {
      FileLoader loader = new FileLoader();
      AlignmentOrder order = new AlignmentOrder(alignment);
      AlignFrame newAFrame = loader.LoadFileWaitTillLoaded(
              HMMERCommands.ALIGNMENTINPUT, DataSourceType.FILE);
      Map<Integer, SequenceI> copy = new HashMap<>(
              hmmSeqs);
      addSeqs(newAFrame, copy);
      SequenceI seq = newAFrame.getViewport().getAlignment()
              .getSequenceAt(0);
      hmm.mapToReferenceAnnotation(newAFrame, seq);
      addSeqs(af, hmmSeqs);
    }
    else
    {
      af.getViewport().getAlignment().getSequences().clear();
      af.setIsRecurring(true);
      af.loadJalviewDataFile(HMMERCommands.ALIGNMENTBUFFER,
              DataSourceType.FILE, FileFormat.Stockholm, null);
      af.setIsRecurring(false);
      addSeqs(af, hmmSeqs);
    }
  }

  private void addSeqs(AlignFrame alignFrame, Map<Integer, SequenceI> map)
  {
    for (Map.Entry<Integer, SequenceI> entry : map.entrySet())
    {
      SequenceI seq = entry.getValue();
      Integer pos = entry.getKey();
      HMMERCommands.addHMMConsensusSequence(alignFrame, seq, pos);
    }
  }

  }


