package jalview.hmmer;

import jalview.datamodel.Alignment;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.HiddenMarkovModel;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.io.HMMFile;
import jalview.io.StockholmFile;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.List;

public class HMMERCommands
{
  // Path of hmmer binaries directory
  static String HMMERFOLDER = "/Documents/";

  static final String JALVIEWDIRECTORY = System.getProperty("user.dir")
          + "/";

  static String HMMALIGN = "/hmmalign ";

  static String HMMBUILD = "/hmmbuild ";

  static String HMMSEARCH = "/hmmsearch ";

  static final String HMMBUFFER = "src/jalview/hmmer/hmm_buffer.hmm ";

  static final String ALIGNMENTBUFFER = "src/jalview/hmmer/alignment_buffer.sto ";

  static final String ALIGNMENTINPUT = "src/jalview/hmmer/alignment_input.sto ";

  static final String OUTPUTALIGNMENT = "-o " + JALVIEWDIRECTORY
          + ALIGNMENTINPUT;

  static final String NAME = "-n ";

  static final String SPACE = " ";

  static final String ALLCOL = "--allcol ";

  static final String TRIM = "--trim ";


  /**
   * Runs a command in the terminal.
   * 
   * @param command
   * @throws IOException
   * @throws InterruptedException
   */
  public static void runCommand(String command)
          throws IOException, InterruptedException
  {
    final Process p = Runtime.getRuntime().exec(command);

    new Thread(new Runnable()
    {
      @Override
      public void run()
      {
        BufferedReader input = new BufferedReader(
                new InputStreamReader(p.getInputStream()));
        String line = null;

        try
        {
          while ((line = input.readLine()) != null)
          {
            System.out.println(line);
          }
        } catch (IOException e)
        {
          e.printStackTrace();
        }
      }
    }).start();

    p.waitFor();
  }

  /**
   * Exports an alignment to the buffer location in Jalview.
   * 
   * @param alignment
   * @throws IOException
   */
  public static void exportData(AlignmentI alignment,
          boolean exportAlignment, boolean exportHMM, HiddenMarkovModel hmm)
          throws IOException
  {
    if (exportAlignment)
    {
      if (!alignment.isAligned())
      {
        alignment.padGaps();
      }
      List<SequenceI> list = alignment.getSequences();
      SequenceI[] array = new SequenceI[list.size()];
      list.toArray(array);
      StockholmFile file = new StockholmFile(new Alignment(array));
      file.setSeqs(array);
      String output = file.print();
      PrintWriter writer = new PrintWriter(ALIGNMENTBUFFER);
      writer.println(output);
      writer.close();
    }

    if (exportHMM)
    {
      HMMFile file = new HMMFile(hmm);
      file.exportFile(HMMBUFFER);
    }
  }

  public static void addHMMConsensusSequence(AlignFrame af, SequenceI seq,
          Integer position)
  {
    seq.getHMM().initHMMSequence(af, position);
    AlignmentI al = af.getViewport().getAlignment();
    af.getViewport().setAlignment(al);
    af.alignPanel.adjustAnnotationHeight();
    af.getViewport().updateSequenceIdColours();
    af.buildSortByAnnotationScoresMenu();
  }


}
